/*
 *  RegExTextField is actually a proxy object for an HTML 'input'.  This object
 *    supports a good and bad icon as well as an error message.
 *
 *  The icon can be positioned on the left or right, while the error message can
 *    be positioned top, right, bottom or left. This position is controled in the
 *    Java class 'IntegerTextFieldPeer.java'.  The behavior is controlled with this
 *    JavaScript.
 *  
 */


/*
 *  Constructor
*/

RegExTextField = function(elementId) {
    this.elementId = elementId;
};



/*
 *  Initialize:
 *      - wire up event listeners
*/


RegExTextField.prototype.init = function() {

    var element = this.getElement();
    var input = this.getInput();
    
    if (!this.enabled) {
        element.readOnly = true;
    }

    
    if (this.text) {
        input.value = this.text;
    }


    if (this.horizontalScroll != 0) {
        input.scrollLeft = this.horizontalScroll;
    }

    this.valid = undefined;

    if(this.statusMsg != null) {
        var messageSpan = document.getElementById(this.elementId + ".msg");
        messageSpan.innerHTML = this.statusMsg;
    }
        
    
    if (this.verticalScroll != 0) {

        if (EchoClientProperties.get("quirkIERepaint")) {

            // Avoid IE quirk where browser will fail to set scroll bar position.

            var originalWidth = element.style.width;
            var temporaryWidth = parseInt(element.clientWidth) - 1;

            input.style.width = temporaryWidth + "px";
            input.style.width = originalWidth;
        }

        input.scrollTop = this.verticalScroll;
    }

    
    if (EchoClientProperties.get("quirkMozillaTextInputRepaint")) {

        // Avoid Mozilla quirk where text will be rendered outside of text field
        // (this appears to be a Mozilla bug).

        var noValue = !input.value;
        if (noValue) {
            input.value = "-";
        }

        var currentWidth = input.style.width;
        input.style.width = "20px";
        input.style.width = currentWidth;

        if (noValue) {
            input.value = "";
        }
    }
    
    
    // Event handlers: 'ActiveTextField.processXXX' refers to a static JS function
    
    EchoEventProcessor.addHandler(element, "blur", "ActiveTextField.processBlur");
    EchoEventProcessor.addHandler(element, "focus", "ActiveTextField.processFocus");
    EchoEventProcessor.addHandler(element, "keyup", "ActiveTextField.processKeyUp");
    
    EchoDomUtil.addEventListener(input, "keypress", ActiveTextField.processKeyPress, false);
    
    EchoDomPropertyStore.setPropertyValue(input, "component", this);
    EchoDomPropertyStore.setPropertyValue(element, "component", this);

    if(this.text)
        this.validateText();


};




/*
 *  Dispose - disconnects event handlers & listeners; removes it's
 *    properties, then deletes the element
*/


RegExTextField.prototype.dispose = function() {

    var element = this.getElement();
    var input = this.getInput();
    
    EchoEventProcessor.removeHandler(element, "blur");
    EchoEventProcessor.removeHandler(element, "focus");
    EchoEventProcessor.removeHandler(element, "keyup");

    EchoDomUtil.removeEventListener(input, "keypress", ActiveTextField.processKeyPress, false);    

    // Remove any updates to text component that occurred during client/server transaction.

    EchoClientMessage.removePropertyElement(element.id, "text");
    
    EchoDomPropertyStore.dispose(input);
    EchoDomPropertyStore.dispose(element);
};


/*
 *  Element accessor
*/

RegExTextField.prototype.getElement = function() {
    return document.getElementById(this.elementId);
};

RegExTextField.prototype.getInput = function() {
    return document.getElementById(this.elementId + ".input");
};


/**
 * Processes a user "action request" on the text component i.e., the pressing
 * of the ENTER key when the the component is focused.
 * If any server-side <code>ActionListener</code>s are registered, an action
 * will be set in the ClientMessage and a client-server connection initiated.
 */

RegExTextField.prototype.doAction = function() {

    if (!this.serverNotify) {
        return;
    }
    
    if (!this.enabled || !EchoClientEngine.verifyInput(this.getInput(), false)) {
        // Don't process actions when client/server transaction in progress.
        EchoDomUtil.preventEventDefault(echoEvent);
        return;
    }
    
    this.updateClientMessage();

    EchoClientMessage.setActionValue(this.elementId, "action");
    EchoServerTransaction.connect();
};


RegExTextField.prototype.doValidationTransition = function() {

    if (!this.serverNotify) {
        return;
    }
    
    if (this.valid == undefined || this.valid == null) {
        return;
    }
    
    
    //this.updateClientMessage();

    EchoClientMessage.setActionValue(this.elementId, "validation-transition", (this.valid) ? "true" : "false");
    EchoServerTransaction.connect();
};



// Prototype event handlers ===================================================================


/**
 * Processes a focus blur event:
 * Records the current state of the text field to the ClientMessage.
 *
 * @param echoEvent the event, preprocessed by the 
 *        <code>EchoEventProcessor</code>
 */

RegExTextField.prototype.processBlur = function(echoEvent) {

    if (!this.enabled || !EchoClientEngine.verifyInput(this.getElement())) {
        return;
    }

    this.validateText();
    this.updateClientMessage();

    EchoFocusManager.setFocusedState(this.elementId, false);
};


/**
 * Processes a focus event:
 * Notes focus state in ClientMessage.
 *
 * @param echoEvent the event, preprocessed by the 
 *        <code>EchoEventProcessor</code>
 */

RegExTextField.prototype.processFocus = function(echoEvent) {
    if (!this.enabled || !EchoClientEngine.verifyInput(this.getInput())) {
        return;
    }
    
    EchoFocusManager.setFocusedState(this.elementId + ".input", true);
};


/**
 * Processes a key press event:
 * Initiates an action in the event that the key pressed was the
 * ENTER key.
 *
 * @param e the DOM Level 2 event
 */

RegExTextField.prototype.processKeyPress = function(e) {
    if (!this.enabled || !EchoClientEngine.verifyInput(this.getElement(), true)) {
        EchoDomUtil.preventEventDefault(e);
        return;
    }
    if (e.keyCode == 13) {
        this.doAction();
    }
};

/**
 * Processes a key up event:
 * Records the current state of the text field to the ClientMessage.
 *
 * @param echoEvent the event, preprocessed by the 
 *        <code>EchoEventProcessor</code>
 */
RegExTextField.prototype.processKeyUp = function(echoEvent) {

    var input = this.getInput();
    
    if (!this.enabled || !EchoClientEngine.verifyInput(input, true)) {
        EchoDomUtil.preventEventDefault(echoEvent);
        return;
    }
    
    this.validateText();
    
    if (this.maximumLength >= 0) {
        if (input.value && input.value.length > this.maximumLength) {
            input.value = input.value.substring(0, this.maximumLength);
        }
    }
    
    this.updateClientMessage();
};

// End: Event Hanlders ========================================================================


RegExTextField.prototype.validateText = function () {

    var input = this.getInput();
    var newval = input.value;

    if (this.regexpFilter != null) {
      var cleanval = "";
      
      for (i = 0; i < newval.length; i++){
        if (this.regexpFilter.test(cleanval + newval.substr(i,1))) {
          cleanval = cleanval + newval.substr(i,1);
        }
      }
      input.value = cleanval;
    }

    newval = input.value;

    var status = false;

    if (this.regexp != null && (newval != null)) {
        status = this.regexp.test(newval);    
    } else {
        status = true;
    }

    var goodIcon = document.getElementById(this.elementId + ".good");
    var errorIcon = document.getElementById(this.elementId + ".error");
    var messageSpan = document.getElementById(this.elementId + ".msg");

    if(newval.length == 0) {
        input.style.color = this.foreground;
        input.style.background = this.background;
        goodIcon.style.display = (this.isIconAlwaysVisible) ? "" : "none";
        errorIcon.style.display =  "none";
        messageSpan.style.display = (this.isMessageAlwaysVisible) ? "" : "none";
        return;
    }
    
    // First condition means value is in range
    if(status == true) {
        input.style.color = "#000000";
        goodIcon.style.display = (this.isIconVisible) ? "" : "none";
        errorIcon.style.display = "none";
        messageSpan.style.display = (this.isMessageAlwaysVisible) ? "" : "none";

        if(this.valid == undefined || this.valid == false) {
            this.valid = true;
            this.doValidationTransition();
        }

    } else {
        input.style.color = "#ff0000";
        goodIcon.style.display = "none";
        errorIcon.style.display = (this.isIconVisible) ? "" : "none";
        messageSpan.style.display = (this.isMessageAlwaysVisible || this.isMsgVisible) ? "" : "none";
        
        if(this.valid == undefined || this.valid == true) {
            this.valid = false;
            this.doValidationTransition();
        }
    }
    
  };


/**
 * Updates the component state in the outgoing <code>ClientMessage</code>.
 */

RegExTextField.prototype.updateClientMessage = function() {
    var input = this.getInput();
    var textPropertyElement = EchoClientMessage.createPropertyElement(this.elementId, "text");
    
    if (textPropertyElement.firstChild) {
        textPropertyElement.firstChild.nodeValue = input.value;
    } else {
        textPropertyElement.appendChild(EchoClientMessage.messageDocument.createTextNode(input.value));
    }
    
    //EchoClientMessage.setPropertyValue(this.elementId, "horizontalScroll", element.scrollLeft);
    //EchoClientMessage.setPropertyValue(this.elementId, "verticalScroll", element.scrollTop);
};




// MessageProcessor composition object and methods ============================================

/**
 * Static object/namespace for Text Component MessageProcessor 
 * implementation.
 */

RegExTextField.MessageProcessor = function() { };


/**
 * MessageProcessor process() implementation 
 * (invoked by ServerMessage processor).
 *
 * @param messagePartElement the <code>message-part</code> element to process.
 */

RegExTextField.MessageProcessor.process = function(messagePartElement) {

    for (var i = 0; i<messagePartElement.childNodes.length; ++i) {
    
        if (messagePartElement.childNodes[i].nodeType == 1) {  // Node.ELEMENT_NODE = 1
        
            switch (messagePartElement.childNodes[i].tagName) {
            case "init":
                RegExTextField.MessageProcessor.processInit(messagePartElement.childNodes[i]);
                break;
                
            case "dispose":
                ActiveTextField.MessageProcessor.processDispose(messagePartElement.childNodes[i]);
                break;
                
            case "set-text":
                ActiveTextField.MessageProcessor.processSetText(messagePartElement.childNodes[i]);
                break;

            case "set-foreground":
                ActiveTextField.MessageProcessor.processSetForeground(messagePartElement.childNodes[i]);
                break;
                
            case "set-background":
                ActiveTextField.MessageProcessor.processSetBackground(messagePartElement.childNodes[i]);
                break;
                
            }
        }
    }
};




/**
 * Processes an <code>init</code> message to initialize the state of a 
 * Text Component that is being added.
 *
 * @param initMessageElement the <code>init</code> element to process
 */

RegExTextField.MessageProcessor.processInit = function(initMessageElement) {


    for (var item = initMessageElement.firstChild; item; item = item.nextSibling) {
    
        var elementId = item.getAttribute("eid");        
        var textComponent = new RegExTextField(elementId);
        
        textComponent.enabled       = item.getAttribute("enabled") != "false";
        textComponent.text          = item.getAttribute("text") ? item.getAttribute("text") : "";

        textComponent.serverNotify  = item.getAttribute("server-notify") == "true";
        textComponent.maximumLength = item.getAttribute("maximum-length") ? item.getAttribute("maximum-length") : -1;
        
        textComponent.horizontalScroll = item.getAttribute("horizontal-scroll") 
                                         ? parseInt(item.getAttribute("horizontal-scroll")) : 0;
        textComponent.verticalScroll = item.getAttribute("vertical-scroll") 
                                         ? parseInt(item.getAttribute("vertical-scroll")) : 0;
             
        textComponent.isIconVisible = (item.getAttribute("icon-visible") == "true") ? true : false;
        textComponent.isMsgVisible =  (item.getAttribute("msg-visible")  == "true") ? true : false;
        textComponent.isMessageAlwaysVisible = (item.getAttribute("msg-always-visible") == "true") ? true : false;
        textComponent.isIconAlwaysVisible = (item.getAttribute("icon-always-visible") == "true") ? true : false;

        textComponent.statusMsg = item.getAttribute("status-msg");


        // RegExTextField specific values
        textComponent.regexpFilter  = item.getAttribute("regexp-filter") ? new RegExp(item.getAttribute("regexp-filter")) : null;
        textComponent.regexp  = item.getAttribute("regexp") ? new RegExp(item.getAttribute("regexp")) : null;
                
        textComponent.init();
    }
};


